@@ -1,10 +1,15 @@
 /* 
- * Copyright (c) 2001 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2003 QUALCOMM Incorporated.  All rights reserved.
  * The file license.txt specifies the terms for use, modification,
  * and redistribution.
  *
  * Revisions:
  *
+ *     11/27/01  [rcg]
+ *              - Worked around a bug (since fixed) in OpenBSD by
+ *                setting file descriptor to be blocking *before*
+ *                we fork, and ignoring what fcntl() says it is.
+ *
  *     04/03/01  [rcg]
  *              - Be a little nicer with "-v".
  *
@@ -540,7 +545,7 @@ main ( int argc, char *argv[] )
      * blocking there.
      */
     fd_flags = fcntl ( sockfd, F_GETFL, 0 );
-    rslt     = fcntl ( sockfd, F_SETFL, O_NONBLOCK | fd_flags );
+    rslt     = fcntl ( sockfd, F_SETFL, fd_flags | O_NONBLOCK );
     if ( rslt == -1 )
         err_dump ( HERE, "Unable to set sockfd(%d) to be non-blocking",
                    sockfd );
@@ -893,6 +898,21 @@ motherforker ( int newsockfd, int sockfd )
 
     TRACE ( trace_file, POP_DEBUG, HERE, "new connection; fd=%d", newsockfd );
 
+    /*
+     * Make sure we pass a blocking socket to Qpopper.  Do this before we
+     * fork, and even if fcntl() insists the socket is already blocking,
+     * because of a bug in older versions of OpenBSD.
+     */
+    fd_flags = fcntl ( newsockfd, F_GETFL, 0 );
+    TRACE ( trace_file, POP_DEBUG, HERE, "newsockfd (%d) flags: %#x",
+            newsockfd, fd_flags );
+    rslt = fcntl ( newsockfd, F_SETFL, fd_flags & ~O_NONBLOCK );
+    if ( rslt == -1 )
+        err_dump ( HERE, "Unable to set newsockfd (%d) to be blocking",
+                   newsockfd );
+    TRACE ( trace_file, POP_DEBUG, HERE, "set fd %d blocking (%#x)",
+            newsockfd, fcntl ( newsockfd, F_GETFL, 0 ) );
+
 #ifndef _DEBUG
     childpid = fork();
     if ( childpid < 0 )
@@ -917,23 +937,6 @@ motherforker ( int newsockfd, int sockfd )
 
 #endif /* not _DEBUG */
 
-        /*
-         * Make sure we pass a blocking socket to Qpopper
-         */
-        fd_flags = fcntl ( newsockfd, F_GETFL, 0 );
-        TRACE ( trace_file, POP_DEBUG, HERE, "newsockfd (%d) flags: %#x",
-                 newsockfd, fd_flags );
-        if ( fd_flags & O_NONBLOCK )
-        {
-            
-            rslt = fcntl ( newsockfd, F_SETFL, fd_flags - O_NONBLOCK );
-            if ( rslt == -1 )
-                err_dump ( HERE, "Unable to set newsockfd (%d) to be blocking",
-                           newsockfd );
-            TRACE ( trace_file, POP_DEBUG, HERE, "set fd %d blocking (%#x)",
-                    newsockfd, fcntl ( newsockfd, F_GETFL, 0 ) );
-        }
-
         dup2    ( newsockfd, 0 );
         dup2    ( newsockfd, 1 );
         dup2    ( newsockfd, 2 );
